@@ -74,7 +74,10 @@ def initialise_bot(topLevelConfig, robotFolder, prefix, path_index):
             robot._follow_collider_offset = config.get("follow_collider", [0, 0])
             ScriptLoader.instance.robots[prefix] = robot
             ScriptLoader.instance.outstanding_events[prefix] = []
-            scriptname = config.get("script", "code.py")
+            if config.get("type", "python") == "mindstorms":
+                scriptname = config.get("script", "program.ev3")
+            else:
+                scriptname = config.get("script", "code.py")
             if scriptname is not None:
                 scriptname = find_abs(scriptname, code_locations(robotFolder))
             ScriptLoader.instance.scriptnames[prefix] = scriptname
@@ -120,9 +123,19 @@ class RobotInteractor(IInteractor):
         for port, device in self.devices.items():
             if device.device_type not in res:
                 res[device.device_type] = {}
-            res[device.device_type][device._getObjName(port)] = device.toObject()
+            try:
+                res[device.device_type][device._getObjName(port)] = device.toObject()
+            except Exception as e:
+                # Something has failed, so let the attached script know.
+                return str(e)
         return res
 
+    def resetBot(self):
+        ScriptLoader.instance.object_map[self.robot_key].body.velocity = (0, 0)
+        ScriptLoader.instance.object_map[self.robot_key].body.angular_velocity = 0
+        for dev in getattr(self, "devices", {}).values():
+            dev.reset()
+
 
 class Robot:
     """
