@@ -1,5 +1,4 @@
 preset_locations = lambda: ["workspace/presets/", "workspace", "package/presets/"]
-bot_locations = lambda: ["workspace/robots/", "workspace", "package/examples/robots/"]
 config_locations = lambda: ["workspace", "package"]
 device_locations = lambda: ["workspace/devices/", "package/devices/"]
 theme_locations = lambda: ["workspace/assets/", "workspace", "package/assets"]
@@ -33,3 +32,18 @@ def batch_locations():
             if os.path.isdir(os.path.join(custom_path, name)):
                 locations.append(f"workspace/custom/{name}/")
     return locations
+
+
+def bot_locations():
+    import os
+    from ev3sim.file_helper import find_abs_directory
+    from ev3sim.simulation.loader import StateHandler
+
+    locations = ["workspace/robots/", "package/examples/robots/", "workspace"]
+    if StateHandler.WORKSPACE_FOLDER:
+        custom_path = find_abs_directory("workspace/custom/", create=True)
+        for name in os.listdir(custom_path):
+            if os.path.isdir(os.path.join(custom_path, name)):
+                # Favour custom locations over the catch-all workspace entry.
+                locations.insert(2, f"workspace/custom/{name}/")
+    return locations
